from xclingo import Explainer as Explainer
from xclingo import __version__ as xclingo_version
from clingo import Control
from argparse import ArgumentParser, FileType
import sys

def check_options():
    # Handles arguments of xclingo
    parser = ArgumentParser(description='Tool for explaining (and debugging) ASP programs', prog='xclingo')
    parser.add_argument('--version', action='version',
                        version='xclingo {version}'.format(version=xclingo_version),
                        help='Prints the version and exists.')
    optional_group = parser.add_mutually_exclusive_group()
    optional_group.add_argument('--only-translate', action='store_true',
                        help="Prints the internal translation and exits.")
    optional_group.add_argument('--only-translate-annotations', action='store_true',
                        help="Prints the internal translation and exits.")
    optional_group.add_argument('--only-explanation-atoms', action='store_true',
                        help="Prints the atoms used by the explainer to build the explanations.")
    parser.add_argument('--auto-tracing', type=str, choices=["none", "facts", "all"], default="none",
                        help="Automatically creates traces for the rules of the program. Default: none.")
    parser.add_argument('-n', nargs=2, default=(1,1), type=int, help="Number of answer sets and number of desired explanations.")
    parser.add_argument('infiles', nargs='+', type=FileType('r'), default=sys.stdin, help="ASP program")
    return parser.parse_args()

def read_files(files):
    return "\n".join([file.read() for file in files])

def translate(program, auto_trace):
    explainer = Explainer(auto_trace=auto_trace)
    explainer.add('base', [], program)
    explainer._translate_program()
    translation =  explainer._preprocessor.get_translation()
    translation += explainer._getExplainerLP(auto_trace=auto_trace)
    return translation   

def print_explanation_atoms(control, explainer: Explainer):
    with control.solve(yield_=True) as it:
        nanswer=1
        for m in it:
            print(f'Answer {nanswer}')
            print(' '.join([str(sym) for sym in m.symbols(shown=True)]))
            nexpl=1
            for xclingo_m in explainer.get_xclingo_models(m):
                print(f'Explanation: {nexpl}')
                print("\n".join([str(sym) for sym in xclingo_m.symbols(shown=True)]))
                print()
                nexpl+=1
            nanswer+=1

def print_text_explanations(control, explainer:Explainer):
    with control.solve(yield_=True) as it:
        nanswer=1
        for m in it:
            print(f'Answer {nanswer}')
            for e in explainer.explain(m):
                print(e.ascii_tree())
            nanswer+=1

def main():
    args = check_options()

    if args.only_translate_annotations:
        program = read_files(args.infiles)
        from xclingo.preprocessor import Preprocessor
        print(Preprocessor.translate_annotations(program))
        return 0

    if args.only_translate:
        program = read_files(args.infiles)
        print(translate(program, args.auto_tracing))
        return 0

    control = Control([str(args.n[0])])
    explainer = Explainer(
        [
            str(args.n[1]), 
        ], 
        auto_trace=args.auto_tracing
    )

    for file in args.infiles:
        prog = file.read()
        explainer.add(file.name, [], prog)
        control.add("base", [], prog)

    control.ground([("base", [])])

    if args.only_explanation_atoms:
        print_explanation_atoms(control, explainer)
    else:
        print_text_explanations(control, explainer)    

if __name__ == '__main__':
    main()



from typing import Iterable
from clingo import Symbol

class Explanation:

    @staticmethod
    def from_model(symbols:Iterable[Symbol]):
        table = dict()
        for s in symbols:
            parent = str(s.arguments[0])
            child = str(s.arguments[1])
            child_item = table.get(child, None)
            parent_item = table.get(parent, None)

            if child_item is None:
                child_item = ExplanationNode()
                table[child] = child_item
            child_item.add_label(str(s.arguments[2]).strip('"'))
            
            if parent_item is None:
                parent_item = ExplanationRoot(explanation_atoms=symbols) if parent == 'root' else ExplanationNode()
                parent_item.add_cause(child_item)
                table[parent] = parent_item
            
            if child_item not in parent_item.causes:
                parent_item.add_cause(child_item)
        
        return table['root']

    @staticmethod
    def ascii_branch(level):
        if level > 0:
            return "  |" * (level) + "__"
        else:
            return ""
    
    def preorder_iterator(self):
        stack = [iter([self])]
        level = 0
        while (stack):
            try:
                current = next(stack[-1])
                yield (current, level)
                stack.append(iter(current.causes))
                level += 1
            except StopIteration:
                stack.pop()
                level += -1

    def ascii_tree(self):
        expl = ""
        for node, level in self.preorder_iterator():
            expl += "{branch}{text}\n".format(
                branch=Explanation.ascii_branch(level),
                text=node.get_node_text(),
            )
        return expl

    def is_equal(self, other):
        if not isinstance(other, Explanation):
            return False

        for (node1, level1), (node2, level2) in zip(self.preorder_iterator(), other.preorder_iterator()):
            if not node1._node_equals(node2):
                return False

            if (level1 != level2):
                return False
        
        return True

    def add_cause(self, cause):
        self.causes.append(cause)

    def add_label(self, label):
        self.labels.append(label)

class ExplanationRoot(Explanation):

    def __init__(self, causes=None, explanation_atoms=None):
        self.causes = list() if causes is None else causes
        self._explanation_atoms = explanation_atoms

    def get_node_text(self):
        return "  *"

    def _node_equals(self, other):
        if not isinstance(other, ExplanationRoot):
            return False

        return True

class ExplanationNode(Explanation):
    """
    A non-binary tree.
    """

    def __init__(self, labels=None, causes=None):
        self.labels = list() if labels is None else labels
        self.causes = list() if causes is None else causes

    def get_node_text(self):
        return ";".join(self.labels)

    def _node_equals(self, other):
        if not isinstance(other, ExplanationNode):
            return False

        if self.label.replace_values() != other.label.replace_values():
            return False

        return True



from typing import Iterable
from clingo import Model, Function, String
from clingo.ast import ProgramBuilder, parse_string
from clingo.control import Control
from clingo.symbol import SymbolType
from xclingo.explanation import Explanation
from xclingo.preprocessor import Preprocessor

from clingo.core import MessageCode

class Context:
    def label(self, text, tup):
        if text.type == SymbolType.String:
            text = text.string
        else:
            text = str(text).strip('"')
        for val in tup.arguments:
            text = text.replace("%", val.string if val.type==SymbolType.String else str(val), 1)
        return [String(text)]

    def inbody(self, body):
        if len(body.arguments)>0:
            return [Function(
                '',
                [a, body],
                True,
            )
            for a in body.arguments]
        else:
            return Function('empty', [], True)

class Explainer():
    def __init__(self, internal_control_arguments=['1'], auto_trace="none"):
        self._preprocessor = Preprocessor()
        self._memory = []
        
        self._internal_control_arguments = internal_control_arguments 
        self._auto_trace = auto_trace
        self._translated = False
        self._current_model = []

        self._no_labels = False
        self._no_show_trace = False

    def logger(self, _code, msg):
        if _code == MessageCode.AtomUndefined:
            if 'xclingo_muted(Cause)' in msg:
                return
            if '_xclingo_label_tree/3' in msg:
                return
            if '_xclingo_label' in msg:
                self._no_labels = True
                return
            if '_xclingo_show_trace' in msg:
                self._no_show_trace = True
        print(msg)

    def print_messages(self):
        if self._no_labels:
            print('xclingo info: any atom or rule has been labelled.')
        if self._no_show_trace:
            print('xclingo info: any atom has been affected by a %!show_trace annotation.')
    
    def clean_log(self):
        self._no_labels = False
        self._no_show_trace = False

    def _getExplainerLP(self, auto_trace="none"):
        if hasattr(self, '_explainerLP') == False:
            setattr(self, '_explainerLP', self._loadExplainerLP(auto_trace))
        return self._explainerLP

    def _loadExplainerLP(self, auto_trace="none"):
        try:
            import importlib.resources as pkg_resources
        except ImportError:
            # Try backported to PY<37 `importlib_resources`.
            import importlib_resources as pkg_resources

        from . import xclingo_lp  # relative-import the *package* containing the templates
        program = pkg_resources.read_text(xclingo_lp, 'xclingo.lp')
        if auto_trace == "all":
            program += pkg_resources.read_text(xclingo_lp, 'autotrace_all.lp')
        elif auto_trace == "facts":
            program += pkg_resources.read_text(xclingo_lp, 'autotrace_facts.lp')
        return program

    def add(self, program_name:str, parameters: Iterable[str], program:str):
        self._memory.append((program_name, program))

    def _initialize_control(self):
        return Control(
            self._internal_control_arguments + \
                [
                    '--project=project'
                ], 
            logger=self.logger)

    def _translate_program(self):
        self._preprocessor._rule_count = 1
        for name, program in self._memory:
            self._preprocessor.translate_program(program, name=name)

    def _ground(self, control, model, context=None):
        if not self._translated:
            self._translate_program()
            self._translated
            
        with ProgramBuilder(control) as builder:
            parse_string(
                self._getExplainerLP(auto_trace=self._auto_trace)+self._preprocessor.get_translation(),
                lambda ast: builder.add(ast),
            )
        
        with control.backend() as backend:
            for sym in model.symbols(shown=True):
                atm_id = backend.add_atom(Function('_xclingo_model', [sym], True))
                backend.add_rule([atm_id], [], False)
            
        control.ground([('base', [])], context=context if context is not None else Context())


    def _get_explanations(self, control):
        with control.solve(yield_=True) as it:
            for expl_model in it:
                syms = expl_model.symbols(shown=True)
                if len(syms)>0:
                    yield Explanation.from_model(syms)
    
    def _get_models(self, control):
        with control.solve(yield_=True) as it:
            for expl_model in it:
                yield expl_model

    def get_xclingo_models(self, model:Model) -> Iterable[Explanation]:
        control = self._initialize_control()
        self.clean_log()
        self._ground(control, model)
        self.print_messages()
        return self._get_models(control)

    def explain(self, model:Model, context=None) -> Iterable[Explanation]:
        control = self._initialize_control()    
        self.clean_log()
        self._ground(control, model, context)
        self.print_messages()
        return self._get_explanations(control)


class XclingoControl:
    def __init__(self, n_solutions='1', n_explanations='1', auto_trace='none'):
        self.n_solutions = n_solutions
        self.n_explanations = n_explanations

        self.control = Control([n_solutions if type(n_solutions)==str else str(n_solutions)])
        self.explainer = Explainer(
            [
                n_explanations if type(n_explanations)==str else str(n_explanations), 
            ], 
            auto_trace=auto_trace
        )

        self._explainer_context = None

    def add(self, name, parameters, program):
        self.explainer.add(name, [], program)
        self.control.add("base", parameters, program)

    def ground(self, context=None, explainer_context=None):
        self.control.ground([("base", [])], context)

        self._explainer_context = explainer_context

    def explain(self):
        with self.control.solve(yield_=True) as it:
            nanswer=1
            for m in it:
                print(f'Answer {nanswer}')
                for e in self.explainer.explain(m, context=self._explainer_context):
                    print(e.ascii_tree())
                nanswer+=1



from clingo.symbol import Number
from ._utils import translate_show_all, translate_trace, translate_trace_all, translate_mute, \
    is_xclingo_label, is_xclingo_show_trace, is_choice_rule, is_label_rule, is_xclingo_mute, \
    is_constraint
from clingo import ast

class Preprocessor:
    def __init__(self):
        self._rule_count = 1
        self._last_trace_rule = None
        self._translation = ""
    
    def increment_rule_count(self):
        n = self._rule_count
        self._rule_count += 1
        return n

    @staticmethod
    def translate_annotations(program):
        return translate_trace_all(
            translate_show_all(
                translate_trace(
                    translate_mute(
                        program
                    )
                )
            )
        )

    def propagates(self, lit_list):
        for lit in lit_list:
            if lit.sign == ast.Sign.NoSign and lit.atom.ast_type == ast.ASTType.SymbolicAtom:
                yield lit        

    def sup_body(self, lit_list):
        loc = ast.Location(
                ast.Position("",0,0),
                ast.Position("",0,0),
            )
        for lit in lit_list:
            if lit.atom.ast_type == ast.ASTType.SymbolicAtom:
                yield ast.Literal(
                    loc,
                    lit.sign,
                    ast.SymbolicAtom(
                        ast.Function(
                            loc,
                            "_xclingo_model",
                            [lit.atom.symbol],
                            False,
                            )
                    )
                )
            else:
                yield lit
    
    def sup_head(self, rule_id, rule_ast):
        loc = ast.Location(
                ast.Position("",0,0),
                ast.Position("",0,0),
            )
        head = ast.Literal(
                loc,
                ast.Sign.NoSign,
                ast.SymbolicAtom(
                    ast.Function(
                        loc,
                        "_xclingo_sup",
                        [
                            ast.SymbolicTerm(loc, Number(rule_id)),
                            rule_ast.head.atom,
                            ast.Function( # tuple
                                loc,
                                '', 
                                list(self.propagates(rule_ast.body)),
                                False
                            )
                        ],
                        False,
                    ),
                )
            )
        return head

    def support_rule(self, rule_id, rule_ast):
        loc = ast.Location(
                ast.Position("",0,0),
                ast.Position("",0,0),
            )
        head = self.sup_head(rule_id, rule_ast)
        body = list(self.sup_body(rule_ast.body))

        return ast.Rule(loc, head, body)

    def fbody_head(self, rule_id, rule_ast):
        loc = ast.Location(
                ast.Position("",0,0),
                ast.Position("",0,0),
            )
        head = ast.Literal(
                loc,
                ast.Sign.NoSign,
                ast.SymbolicAtom(
                    ast.Function(
                        loc,
                        "_xclingo_fbody",
                        [
                            ast.SymbolicTerm(loc, Number(rule_id)),
                            rule_ast.head.atom,
                            ast.Function( # tuple
                                loc,
                                '', 
                                list(self.propagates(rule_ast.body)),
                                False
                            )
                        ],
                        False,
                    ),
                )
            )
        return head

    def fbody_body(self, lit_list):
        loc = ast.Location(
                ast.Position("",0,0),
                ast.Position("",0,0),
            )
        for lit in lit_list:
            if lit.atom.ast_type == ast.ASTType.SymbolicAtom:
                if lit.sign == ast.Sign.NoSign:
                    yield ast.Literal(
                        loc,
                        lit.sign,
                        ast.SymbolicAtom(
                            ast.Function(
                                loc,
                                "_xclingo_f_atom",
                                [lit.atom.symbol],
                                False,
                                )
                        )
                    )
                else:
                    yield ast.Literal(
                        loc,
                        ast.Sign.Negation,
                        ast.SymbolicAtom(
                            ast.Function(
                                loc,
                                "_xclingo_model",
                                [lit.atom.symbol],
                                False,
                                )
                        )
                    )
            else:
                yield lit

    def fbody_rule(self, rule_id, rule_ast):
        loc = ast.Location(
                ast.Position("",0,0),
                ast.Position("",0,0),
            )
        head = self.fbody_head(rule_id, rule_ast)
        body = list(self.fbody_body(rule_ast.body))
        return ast.Rule(loc, head, body)

    def label_rule(self, rule_id, label_rule_ast, rule_body):
        loc = ast.Location(
                ast.Position("",0,0),
                ast.Position("",0,0),
            )
        head_var = ast.Variable(loc, 'Head')
        head = ast.Literal(
            loc,
            label_rule_ast.head.sign,
            ast.SymbolicAtom(ast.Function(
                loc,
                label_rule_ast.head.atom.symbol.name,
                [
                    head_var,
                    label_rule_ast.head.atom.symbol.arguments[1]
                ],
                False,
            ))
        )
        body = [
            ast.Literal(
                loc,
                ast.Sign.NoSign,
                ast.SymbolicAtom(ast.Function(
                    loc,
                    '_xclingo_f',
                    [
                        ast.SymbolicTerm(loc, Number(rule_id)),
                        head_var,
                        ast.Function(
                            loc,
                            '',
                            list(self.propagates(rule_body)),
                            False,
                            ),
                    ],
                    False,
                ))
            )
        ]
        rule = ast.Rule(loc, head, body)
        return rule

    def label_atom(self, rule_ast):
        loc = ast.Location(
            ast.Position('', 0, 0),
            ast.Position('', 0, 0),
        )
        fatom = ast.Literal(
            loc,
            ast.Sign.NoSign,
            ast.SymbolicAtom(ast.Function(
                loc,
                '_xclingo_intree',
                [rule_ast.head.atom.symbol.arguments[0]],
                False,
            ))
        )
        body = [fatom] + list(self.sup_body(rule_ast.body))
        rule = ast.Rule(loc, rule_ast.head, body)
        return rule

    def show_trace(self, rule_ast):
        loc = ast.Location(
            ast.Position('', 0, 0),
            ast.Position('', 0, 0),
        )
        literal_head = ast.Literal(
            loc,
            ast.Sign.NoSign,
            ast.SymbolicAtom(rule_ast.head.atom.symbol.arguments[0]),
        )
        rule = ast.Rule(loc, rule_ast.head, list(self.sup_body([literal_head] + list(rule_ast.body))))
        return rule

    def mute(self, rule_ast):
        loc = ast.Location(
            ast.Position('', 0, 0),
            ast.Position('', 0, 0),
        )
        literal_head = ast.Literal(
            loc,
            ast.Sign.NoSign,
            ast.SymbolicAtom(rule_ast.head.atom.symbol.arguments[0]),
        )
        rule = ast.Rule(loc, rule_ast.head, list(self.sup_body([literal_head] + list(rule_ast.body))))
        return rule    

    def add_to_translation(self, a):
        self._translation+=f'{a}\n'

    def add_comment_to_translation(self, a):
        self._translation+=f'% {a}\n'

    def translate_rule(self, rule_ast):
        self.add_comment_to_translation(rule_ast)
        if rule_ast.ast_type == ast.ASTType.Rule and not is_constraint(rule_ast):
            if is_xclingo_label(rule_ast):
                if is_label_rule(rule_ast):
                    self._last_trace_rule = rule_ast
                    return
                self.add_to_translation(self.label_atom(rule_ast))
            elif is_xclingo_show_trace(rule_ast):
                self.add_to_translation(self.show_trace(rule_ast))
                pass
            elif is_xclingo_mute(rule_ast):
                self.add_to_translation(self.mute(rule_ast))
            else:
                rule_id = self.increment_rule_count()
                if is_choice_rule(rule_ast):
                    for cond_lit in rule_ast.head.elements:
                        false_rule = ast.Rule(
                            ast.Location(
                                ast.Position('',0,0),
                                ast.Position('',0,0),
                            ),
                            cond_lit.literal,
                            list(cond_lit.condition) + list(rule_ast.body),
                        )
                        self.add_to_translation(self.support_rule(rule_id, false_rule))
                        self.add_to_translation(self.fbody_rule(rule_id, false_rule))
                        if self._last_trace_rule is not None:
                            self.add_to_translation(self.label_rule(rule_id, self._last_trace_rule, false_rule.body))
                    if self._last_trace_rule is not None:
                        self._last_trace_rule = None
                else:  # Other cases
                    self.add_to_translation(self.support_rule(rule_id, rule_ast))
                    self.add_to_translation(self.fbody_rule(rule_id, rule_ast))
                    if self._last_trace_rule is not None:
                        self.add_to_translation(self.label_rule(rule_id, self._last_trace_rule, rule_ast.body))
                        self._last_trace_rule = None
    
    def translate_program(self, program, name=''):
        self._translation += '%'*8 + name + '%'*8 + '\n'
        ast.parse_string(
            Preprocessor.translate_annotations(program), 
            lambda ast: self.translate_rule(ast),
        )

    def get_translation(self):
        return self._translation



import re
from clingo import ast

def translate_trace(program):
    """
    Replaces the 'label_rule' magic comments in the given program for a version of the rules labelled with theory atoms.
    @param str program: the program that is intended to be modified.
    @return str:
    """
    for hit in re.findall("(%!trace_rule \{(\".*\")(?:,(.*))?\}[ ]*[\n ]*)", program):
        # 0: original match  1: label text  2: label parameters  3: head of the rule  4: body of the rule
        program = program.replace(
            hit[0],
            "{name}(id, @label({text}, ({parameters},) )).\n".format(
                text=hit[1], parameters=hit[2] if hit[2] else "",
                name="_xclingo_label"
            )
        )

    return program


def translate_trace_all(program):
    """
    Replaces the 'label_atoms' magic comments in the given program for label_atoms rule.
    @param str program: the program that is intended to be modified
    @return str:
    """
    for hit in re.findall("(%!trace \{(\".*\")(?:,(.*))?\} (\-?[_a-z][_a-zA-Z0-9]*(?:\((?:[\-\+a-zA-Z0-9 \(\)\,\_])+\))?)(?:[ ]*:[ ]*(.*))?\.)", program):
        # 0: original match 1: "label" 2:v1,v2  3: head  4: body.
        program = program.replace(
            hit[0],
            "{name}({head}, @label({text}, ({parameters},)) ){body}.".format(
                head=hit[3], 
                text=hit[1], 
                parameters=hit[2], 
                body=(" :- " + hit[4]) if hit[4] else "",
                name="_xclingo_label"
                )
        )

    return program


def translate_show_all(program):
    """
    Replaces 'explain' magic comments in the given program for a rule version of those magic comments.
    @param str program:
    @return:
    """
    for hit in re.findall("(%!show_trace ((\-)?([_a-z][_a-zA-Z0-9]*(?:\((?:[\-a-zA-Z0-9 \(\)\,\_])+\))?)(?:[ ]*:[ ]*(.*))?\.))", program):
        # 0: original match  1: rule  2: negative_sign  3: head of the rule  4: body of the rule
        program = program.replace(
            hit[0],
            "{name}({classic_negation}{head}){body}.".format(
                sign="" if not hit[2] else "n",
                name="_xclingo_show_trace",
                head=hit[3],
                classic_negation="" if not hit[2] else "-",
                body=" :- " + hit[4] if hit[4] else "")
        )

    return program

def translate_mute(program):
    """
    Replaces 'explain' magic comments in the given program for a rule version of those magic comments.
    @param str program:
    @return:
    """
    for hit in re.findall("(%!mute ((\-)?([_a-z][_a-zA-Z0-9]*(?:\((?:[\-a-zA-Z0-9 \(\)\,\_])+\))?)(?:[ ]*:[ ]*(.*))?\.))", program):
        # 0: original match  1: rule  2: negative_sign  3: head of the rule  4: body of the rule
        program = program.replace(
            hit[0],
            "{name}({classic_negation}{head}){body}.".format(
                sign="" if not hit[2] else "n",
                name="_xclingo_muted",
                head=hit[3],
                classic_negation="" if not hit[2] else "-",
                body=" :- " + hit[4] if hit[4] else "")
        )

    return program

def is_constraint(rule_ast):
    if rule_ast.ast_type == ast.ASTType.Rule:
        if hasattr(rule_ast.head, 'atom'):
            return  rule_ast.head.atom.ast_type == ast.ASTType.BooleanConstant \
                and rule_ast.head.atom == ast.BooleanConstant(0)
    return False
    

def is_xclingo_label(rule_ast):
    return rule_ast.head.ast_type == ast.ASTType.Literal \
        and rule_ast.head.atom.symbol.ast_type == ast.ASTType.Function \
        and rule_ast.head.atom.symbol.name == "_xclingo_label"

def is_xclingo_show_trace(rule_ast):
    return rule_ast.head.ast_type == ast.ASTType.Literal \
        and rule_ast.head.atom.symbol.ast_type == ast.ASTType.Function \
        and rule_ast.head.atom.symbol.name == "_xclingo_show_trace"

def is_xclingo_mute(rule_ast):
    return rule_ast.head.ast_type == ast.ASTType.Literal \
        and rule_ast.head.atom.symbol.ast_type == ast.ASTType.Function \
            and rule_ast.head.atom.symbol.name == "_xclingo_muted"

def is_label_rule(rule_ast):
    # Precondition: is_xclingo_label(rule_ast) == True
    return str(rule_ast.head.atom.symbol.arguments[0]) == "id"

def is_choice_rule(rule_ast):
    return rule_ast.head.ast_type == ast.ASTType.Aggregate \
        and hasattr(rule_ast.head, 'function') == False



import setuptools
from xclingo import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="xclingo",
    version=__version__,
    author="Brais Muñiz",
    author_email="mc.brais@gmail.com",
    description="Tool for explaining and debugging Answer Set Programs.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bramucas/xclingo2",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords=[
        'logic programming',
        'answer set programming',
    ],
    include_package_data=True,
    python_requires='>=3.6.0',
    install_requires=[
        'clingo>=5.5.0.post3',
        'argparse',
    ],
    packages=['xclingo', 'xclingo.preprocessor', 'xclingo.explanation', 'xclingo.xclingo_lp'],
    entry_points={'console_scripts': ['xclingo=xclingo.__main__:main']})
